@@ -1,5 +1,18 @@
 from .api_pb2 import (  # type: ignore
     BinarySensorStateResponse,
+    BluetoothConnectionsFreeResponse,
+    BluetoothDeviceConnectionResponse,
+    BluetoothDeviceRequest,
+    BluetoothGATTGetServicesDoneResponse,
+    BluetoothGATTGetServicesRequest,
+    BluetoothGATTGetServicesResponse,
+    BluetoothGATTNotifyDataResponse,
+    BluetoothGATTNotifyRequest,
+    BluetoothGATTReadDescriptorRequest,
+    BluetoothGATTReadRequest,
+    BluetoothGATTReadResponse,
+    BluetoothGATTWriteDescriptorRequest,
+    BluetoothGATTWriteRequest,
     BluetoothLEAdvertisementResponse,
     ButtonCommandRequest,
     CameraImageRequest,
@@ -56,6 +69,7 @@ from .api_pb2 import (  # type: ignore
     SensorStateResponse,
     SirenCommandRequest,
     SirenStateResponse,
+    SubscribeBluetoothConnectionsFreeRequest,
     SubscribeBluetoothLEAdvertisementsRequest,
     SubscribeHomeassistantServicesRequest,
     SubscribeHomeAssistantStateResponse,
@@ -193,4 +207,18 @@ MESSAGE_TYPE_TO_PROTO = {
     65: MediaPlayerCommandRequest,
     66: SubscribeBluetoothLEAdvertisementsRequest,
     67: BluetoothLEAdvertisementResponse,
+    68: BluetoothDeviceRequest,
+    69: BluetoothDeviceConnectionResponse,
+    70: BluetoothGATTGetServicesRequest,
+    71: BluetoothGATTGetServicesResponse,
+    72: BluetoothGATTGetServicesDoneResponse,
+    73: BluetoothGATTReadRequest,
+    74: BluetoothGATTReadResponse,
+    75: BluetoothGATTWriteRequest,
+    76: BluetoothGATTReadDescriptorRequest,
+    77: BluetoothGATTWriteDescriptorRequest,
+    78: BluetoothGATTNotifyRequest,
+    79: BluetoothGATTNotifyDataResponse,
+    80: SubscribeBluetoothConnectionsFreeRequest,
+    81: BluetoothConnectionsFreeResponse,
 }
